/*******************************************************************************
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 *   http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 ******************************************************************************/

package org.apache.kato.anttasks;

import org.apache.tools.ant.BuildException;
import org.apache.tools.ant.taskdefs.Javadoc;
import org.apache.tools.ant.types.Path;

/**
 * Wrapper ant task around javadoc task.  Adds in required Doclet 
 *
 */
public abstract class AbstractJavaDocAntTask extends Javadoc{

	private Path docletPath=null;
	private String tagName=null;
	private String docletName=null;
	private Path output=null;
	
	public AbstractJavaDocAntTask(String tagName,Class docletClass) {
		this.tagName=tagName;
		//this.docletName=docletClass.getCanonicalName();

		// Duplicate the above functionality for 1.4
		String className = docletClass.getName();
		if (className.indexOf('$') != -1) {
			className = null;
		}
		this.docletName= className;
	}
	
	public void execute() throws BuildException {
	
		if(output==null) throw new BuildException(tagName+" missing required attribute 'output'");
		if(docletPath==null) throw new BuildException(tagName+" missing required attribute 'docletpath'");
		runJavaDoc();
		super.execute();
	}

	private void runJavaDoc() {
		
		// Configure required doclet 
		System.out.println("running "+docletName);
		
		DocletInfo doclet=createDoclet();
		doclet.setName(docletName);
		Path path=doclet.createPath();
		path.setPath(docletPath.toString());
		
		DocletParam parameter=doclet.createParam();
		parameter.setName("-outfile");
		parameter.setValue(output.toString());
		
	}
	
	public void setOutput(Path output) {
		this.output = output;
	}
	public void setDocletPath(Path docletPath) {
		this.docletPath = docletPath;
	}
}
