/*******************************************************************************
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 *   http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 ******************************************************************************/
package javax.tools.diagnostics.image;

/**
 * This class is used to indicate that corruption has been detected in the image.
 * It may indicate corruption of the image file, or it may indicate that 
 * inconsistencies have been detected within the image file, perhaps caused by
 * a bug in the runtime or application.
 * 
 * It may be encountered in two scenarios:
 * <ul>
 * <li>within a CorruptDataException</li>
 * <li>returned as an element from an Iterator</li> 
 * </ul>
 * 
 * Any iterator in javax.diagnostics may implicitly include one or more CorruptData objects
 * within the list of objects it provides. Normal data may be found after the
 * CorruptData object if the javax.diagnostics implementation is able to recover from the
 * corruption.
 * 
 * @see CorruptDataException
 */
public interface CorruptData {

	/**
	 * Provides a string which describes the corruption
	 * 
	 * @return a descriptive string
	 */
	String toString();
	
	/**
	 * Return an address associated with the corruption.
	 * 
	 * If the corruption is not associated with an address, return null.
	 * 
	 * If the corruption is associated with more than one address, return
	 * the one which best identifies the corruption.
	 * 
	 * @return the address of the corrupted data
	 */
	ImagePointer getAddress();
	
}
