/*******************************************************************************
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 *   http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 ******************************************************************************/
package javax.tools.diagnostics.image;

/**
 * Used to indicate that corruption has been detected in the image.
 * @see CorruptData
 */
public class CorruptDataException extends DiagnosticException {

	private static final long serialVersionUID = 1824138624307972457L;

	private CorruptData corruptData;
	
	/**
	 * Construct a new CorruptDataException for the specified corrupt data
	 * 
	 * @param data the corruptData
	 */
	public CorruptDataException(CorruptData data) {
        super(data==null ? "" : data.toString());

        if (data == null) {
        	throw new IllegalArgumentException("data must not be null");
        }
        corruptData = data;
    }
    
	/**
	 * Get more info about the corrupted data
	 * 
	 * @return the CorruptData object
	 */
	public CorruptData getCorruptData() {
    	return corruptData;
    }
    
}
