/*******************************************************************************
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 *   http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 ******************************************************************************/

package javax.tools.diagnostics.runtime.java;

import javax.tools.diagnostics.image.DataUnavailable;

/**
 * <p>
 * Representation of a Java Variable  
 * <p>
 * This is an experimental addition and may be removed at a later date
 */
public interface JavaVariable {

	
	/**
	 * <p>
	 * The name of the variable.
	 * 
	 * @throws DataUnavailable if the information is not available
	 */
	String getName() throws DataUnavailable;

	/**
	 * <p>
	 *  The local variable's signature in JNI format.
	 * 
	 */
	String getSignature();

	/**
	 * <p>
	 *  The start of the local variable's scope within the bytecode.
	 * 
	 */
	int getStart();

	/**
	 * <p>
	 *  The number of bytes this variables scope covers over the bytecode.
	 * 
	 */
	int getLength();

	/**
	 * <p>
	 *  The local variable slot this variable occupies. Passed to JavaStackFrame.getVariable() to retrieve the contents.
	 * 
	 */
	int getSlot();
	
	/**
	 * <p>
	 *  The value of the variable 
	 * 
	 */
	Object getValue();
	
}
