/*******************************************************************************
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 *   http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 ******************************************************************************/
package test.apache.kato;

import java.io.File;
import java.io.IOException;
import java.util.Iterator;
import java.util.List;

import javax.tools.diagnostics.FactoryRegistry;
import javax.tools.diagnostics.image.CorruptDataException;
import javax.tools.diagnostics.image.DataUnavailable;
import javax.tools.diagnostics.image.Image;
import javax.tools.diagnostics.image.ImageFactory;
import javax.tools.diagnostics.image.ImagePointer;
import javax.tools.diagnostics.image.MemoryAccessException;
import javax.tools.diagnostics.runtime.java.JavaClassLoader;
import javax.tools.diagnostics.runtime.java.JavaHeap;
import javax.tools.diagnostics.runtime.java.JavaMethod;
import javax.tools.diagnostics.runtime.java.JavaMonitor;
import javax.tools.diagnostics.runtime.java.JavaObject;
import javax.tools.diagnostics.runtime.java.JavaReference;
import javax.tools.diagnostics.runtime.java.JavaRuntime;
import javax.tools.diagnostics.runtime.java.JavaThread;
import javax.tools.diagnostics.runtime.java.JavaVMInitArgs;

import junit.framework.TestCase;


public class TestFactoryRegistry extends TestCase {

	
	private static final class MockFactory implements ImageFactory {
		
		private String[] suffixes=null;
		
		public MockFactory(String[] suffixes) {
			this.suffixes=suffixes;
		}
		public Image getImage(File imageFile) throws IOException {
			
			return new MockImage();
		}

		public Image getImage(File imageFile, File metadata)
				throws IOException {

			return null;
		}

		public int getMajorVersion() {

			return 0;
		}

		public int getMinorVersion() {

			return 0;
		}

		public int getModificationLevel() {

			return 0;
		}

		public String[] getValidFileExtensions() {
		
			return suffixes;
		}
		@Override
		public JavaRuntime getJavaRuntime(File file) {
			
			return new MockRuntime();
		}
	}
	private static final class MockRuntime implements JavaRuntime {

		@Override
		public List<JavaMethod> getCompiledMethods() {
			// TODO Auto-generated method stub
			return null;
		}

		@Override
		public List<JavaReference> getHeapRoots() {
			// TODO Auto-generated method stub
			return null;
		}

		@Override
		public List<JavaHeap> getHeaps() {
			// TODO Auto-generated method stub
			return null;
		}

		@Override
		public List<JavaClassLoader> getJavaClassLoaders() {
			// TODO Auto-generated method stub
			return null;
		}

		@Override
		public ImagePointer getJavaVM() throws CorruptDataException {
			// TODO Auto-generated method stub
			return null;
		}

		@Override
		public JavaVMInitArgs getJavaVMInitArgs() throws DataUnavailable,
				CorruptDataException {
			// TODO Auto-generated method stub
			return null;
		}

		@Override
		public List<JavaMonitor> getMonitors() {
			// TODO Auto-generated method stub
			return null;
		}

		@Override
		public JavaObject getObjectAtAddress(ImagePointer address)
				throws CorruptDataException, IllegalArgumentException,
				MemoryAccessException, DataUnavailable {
			// TODO Auto-generated method stub
			return null;
		}

		@Override
		public List<JavaThread> getThreads() {
			// TODO Auto-generated method stub
			return null;
		}

		@Override
		public Object getTraceBuffer(String bufferName, boolean formatted)
				throws CorruptDataException {
			// TODO Auto-generated method stub
			return null;
		}

		@Override
		public String getFullVersion() throws CorruptDataException {
			// TODO Auto-generated method stub
			return null;
		}

		@Override
		public String getVersion() throws CorruptDataException {
			// TODO Auto-generated method stub
			return null;
		}

		@Override
		public File getSource() {
			// TODO Auto-generated method stub
			return null;
		}
		
	}
	private static final class MockImage implements Image {
		public List getAddressSpaces() {
			
			return null;
		}

		public long getCreationTime() throws DataUnavailable {
			
			return 0;
		}

		public String getHostName() throws DataUnavailable,
				CorruptDataException {
			
			return null;
		}

		public Iterator getIPAddresses() throws DataUnavailable {
			
			return null;
		}

		public long getInstalledMemory() throws DataUnavailable {
			
			return 0;
		}

		public int getProcessorCount() throws DataUnavailable {
			
			return 0;
		}

		public String getProcessorSubType() throws DataUnavailable,
				CorruptDataException {
			
			return null;
		}

		public String getProcessorType() throws DataUnavailable,
				CorruptDataException {

			return null;
		}

		public String getSystemSubType() throws DataUnavailable,
				CorruptDataException {

			return null;
		}

		public String getSystemType() throws DataUnavailable,
				CorruptDataException {

			return null;
		}

		@Override
		public File getSource() {
			// TODO Auto-generated method stub
			return null;
		}
	}

	public void testConstructor() {
		
		new FactoryRegistry();
	}
	
	public void testSimpleConstructorHasNoFactories() {
		
		FactoryRegistry reg=new FactoryRegistry();
		ImageFactory[] factories=reg.getFactories();
		
		assertEquals(0,factories.length);
		
	}
	
	public void testNullImageFileNameThrowsError() throws IOException {
		
		FactoryRegistry reg=new FactoryRegistry();
		try {
		reg.getImage(null);
		fail("Expected illegal argument exception");
		
		} catch(IllegalArgumentException e) {
			; // ok
		}
		
	}
	
	public void testNullRutimeFileNameThrowsError() throws IOException {
		
		FactoryRegistry reg=new FactoryRegistry();
		try {
		reg.getJavaRuntime(null);
		fail("Expected illegal argument exception");
		
		} catch(IllegalArgumentException e) {
			; // ok
		}
		
	}
	public void testNoSuffixImageFileNameReturnsNull() throws IOException {
		
		FactoryRegistry reg=new FactoryRegistry();
		File f=new File("testfile");
		
		Image result=reg.getImage(f);
		assertNull(result);
		
	}
public void testNoSuffixRuntimeFileNameReturnsNull() throws IOException {
		
		FactoryRegistry reg=new FactoryRegistry();
		File f=new File("testfile");
		
		JavaRuntime result=reg.getJavaRuntime(f);
		assertNull(result);
		
	}
	public void testDotSuffixFileNameReturnsNull() throws IOException {
		
		FactoryRegistry reg=new FactoryRegistry();
		File f=new File("testfile.");
		
		Image result=reg.getImage(f);
		assertNull(result);
		
	}
	public void testSimpleNoSuffixes() throws IOException {
		
		FactoryRegistry reg=new FactoryRegistry();
		reg.addFactory(new MockFactory(null));
		
		File f=new File("testfile.suffix");
		
		Image result=reg.getImage(f);
		assertNull(result);
		
	}
	public void testSimpleSuffixMatch() throws IOException {
		
		FactoryRegistry reg=new FactoryRegistry();
		String[] suffixes=new String[]{"suffix"};
		
		reg.addFactory(new MockFactory(suffixes));
		
		File f=new File("testfile.suffix");
		
		Image result=reg.getImage(f);
		assertNotNull(result);
		
	}

	public void testSimpleMixedCaseSuffixMatch() throws IOException {
		
		FactoryRegistry reg=new FactoryRegistry();
		String[] suffixes=new String[]{"suFFix"};
		
		reg.addFactory(new MockFactory(suffixes));
		
		File f=new File("testfile.suffix");
		
		Image result=reg.getImage(f);
		assertNotNull(result);
		
	}
	
	public void testTwoChoiceSuffixMatch() throws IOException {
		
		FactoryRegistry reg=new FactoryRegistry();
		String[] suffixes=new String[]{"suFFix","other"};
		
		reg.addFactory(new MockFactory(suffixes));
		
		File f=new File("testfile.other");
		
		Image result=reg.getImage(f);
		assertNotNull(result);
		
		
	}
	
public void testTwoFactorySuffixMatch() throws IOException {
		
		FactoryRegistry reg=new FactoryRegistry();
		
		String[] suffixes=new String[]{"suFFix","other"};
		reg.addFactory(new MockFactory(suffixes));
		
		suffixes=new String[]{"rubarb","CustARD"};
		reg.addFactory(new MockFactory(suffixes));
		
		File f=new File("testfile.cusTard");
		
		Image result=reg.getImage(f);
		assertNotNull(result);
		
		
	}

public void testRuntimeReturnsFileName() {
	
}
}
