/*******************************************************************************
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 *   http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 ******************************************************************************/

package org.apache.kato.jvmti.javaruntime.model;

import java.util.LinkedList;
import java.util.List;

import javax.tools.diagnostics.image.CorruptDataException;
import javax.tools.diagnostics.runtime.java.JavaClass;
import javax.tools.diagnostics.runtime.java.JavaClassLoader;
import javax.tools.diagnostics.runtime.java.JavaObject;

public class JClassLoader implements JavaClassLoader {

	private List<JavaClass> cachedClasses=new LinkedList<JavaClass>();
	private List<JavaClass> definedClasses = null;
	public JObject obj = null;
	
	@Override
	public JavaClass findClass(String name) throws CorruptDataException {
		for (JavaClass c: cachedClasses){
			if (c.getName().equals(name))return c;
		}
		return null;
	}

	public void addClass(JClass c) {				
		cachedClasses.add(c);		
	}

	@Override
	public List<JavaClass> getCachedClasses() {
		if(cachedClasses==null) cachedClasses=new LinkedList<JavaClass>();
		return cachedClasses;
	}

	@Override
	public List<JavaClass> getDefinedClasses() {
		if(definedClasses==null) {
			definedClasses=new LinkedList<JavaClass>();
			for(JavaClass clazz : cachedClasses) {
				try {
					if (this == clazz.getClassLoader()) {
						definedClasses.add(clazz);
					}
				} catch (CorruptDataException e) {
					e.printStackTrace();
				}
			}
		}
		
		return definedClasses;
	}

	@Override
	public JavaObject getObject() throws CorruptDataException {
		return obj;
	}
	
}