/*******************************************************************************
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 *   http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 ******************************************************************************/

package org.apache.kato.jvmti.javaruntime.model;

import java.util.Collections;
import java.util.LinkedList;
import java.util.List;
import java.util.logging.Level;

import javax.tools.diagnostics.image.CorruptDataException;
import javax.tools.diagnostics.image.DataUnavailable;
import javax.tools.diagnostics.image.ImagePointer;
import javax.tools.diagnostics.image.ImageSection;
import javax.tools.diagnostics.image.ImageThread;
import javax.tools.diagnostics.image.MemoryAccessException;
import javax.tools.diagnostics.runtime.java.JavaClass;
import javax.tools.diagnostics.runtime.java.JavaHeap;
import javax.tools.diagnostics.runtime.java.JavaObject;
import javax.tools.diagnostics.runtime.java.JavaStackFrame;
import javax.tools.diagnostics.runtime.java.JavaThread;

import org.apache.kato.jvmti.reader.CLogger;

public class JThread implements JavaThread{
	public JMonitor contendedMonitor=null;
	public long id=0;
	public short priority=0;
	public short daemon=0;
	public int groupID=0;
	public String threadName=null;
	private JObject object = null;
	private int state=0; 

	private List<JavaStackFrame> frames=new LinkedList<JavaStackFrame>();

	private final static int JVMTI_THREAD_STATE_ALIVE = 0x1;
	private final static int JVMTI_THREAD_STATE_TERMINATED = 0x2;
	private final static int JVMTI_THREAD_STATE_RUNNABLE = 0x4;
	private final static int JVMTI_THREAD_STATE_BLOCKED_ON_MONITOR_ENTER = 0x400;
	private final static int JVMTI_THREAD_STATE_WAITING=0x80;
	private final static int JVMTI_THREAD_STATE_WAITING_INDEFINITELY = 0x10;
	private final static int JVMTI_THREAD_STATE_WAITING_WITH_TIMEOUT=0x20;
	private final static int JVMTI_THREAD_STATE_SLEEPING=0x40;
	private final static int JVMTI_THREAD_STATE_IN_OBJECT_WAIT=0x100;
	private final static int JVMTI_THREAD_STATE_PARKED=0x200;
	private final static int JVMTI_THREAD_STATE_SUSPENDED=0x100000;
	private final static int JVMTI_THREAD_STATE_INTERRUPTED=0x200000;
	private final static int JVMTI_THREAD_STATE_IN_NATIVE=0x400000;
	private final static int JVMTI_THREAD_STATE_VENDOR_1=0x10000000;
	private final static int JVMTI_THREAD_STATE_VENDOR_2=0x20000000;
	private final static int JVMTI_THREAD_STATE_VENDOR_3=0x40000000;
	
	public JThread(long id) {
		this.id=id;
	}

	public void setState(int tiState) {
		this.state =0;
		if ((tiState & JVMTI_THREAD_STATE_ALIVE) != 0) {
			state |= JavaThread.STATE_ALIVE;
		}
		
	    if((tiState & JVMTI_THREAD_STATE_TERMINATED) != 0) {
	    	state |= JavaThread.STATE_TERMINATED;
	    }
	    
	    if((tiState & JVMTI_THREAD_STATE_RUNNABLE) != 0) {
	    	state |= JavaThread.STATE_RUNNABLE;
	    }
	    if((tiState & JVMTI_THREAD_STATE_BLOCKED_ON_MONITOR_ENTER) != 0) {
	    	state |= JavaThread.STATE_BLOCKED_ON_MONITOR_ENTER;
		}
	    if((tiState & JVMTI_THREAD_STATE_WAITING) != 0) {
	    	state |= JavaThread.STATE_WAITING;
		}
	    if((tiState & JVMTI_THREAD_STATE_WAITING_INDEFINITELY) != 0) {
	    	state |= JavaThread.STATE_WAITING_INDEFINITELY;
		}
	    if((tiState & JVMTI_THREAD_STATE_WAITING_WITH_TIMEOUT) != 0) {
	    	state |= JavaThread.STATE_WAITING_WITH_TIMEOUT;
		}
	    if((tiState & JVMTI_THREAD_STATE_SLEEPING) != 0) {
	    	state |= JavaThread.STATE_SLEEPING;
		}
	    if((tiState & JVMTI_THREAD_STATE_IN_OBJECT_WAIT) != 0) {
	    	state |= JavaThread.STATE_IN_OBJECT_WAIT;
		}
	    if((tiState & JVMTI_THREAD_STATE_PARKED) != 0) {
	    	state |= JavaThread.STATE_PARKED;
		}
	    if((tiState & JVMTI_THREAD_STATE_SUSPENDED) != 0) {
	    	state |= JavaThread.STATE_SUSPENDED;
		}
	    if((tiState & JVMTI_THREAD_STATE_INTERRUPTED) != 0) {
	    	state |= JavaThread.STATE_INTERRUPTED;
		}
	    if((tiState & JVMTI_THREAD_STATE_IN_NATIVE) != 0) {
	    	state |= JavaThread.STATE_IN_NATIVE;
		}
	    if((tiState & JVMTI_THREAD_STATE_VENDOR_1) != 0) {
	    	state |= JavaThread.STATE_VENDOR_1;
		}
	    if((tiState & JVMTI_THREAD_STATE_VENDOR_2) != 0) {
	    	state |= JavaThread.STATE_VENDOR_2;
		}
	    if((tiState & JVMTI_THREAD_STATE_VENDOR_3) != 0) {
	    	state |= JavaThread.STATE_VENDOR_3;
		}
	}
	
	@Override
	public ImageThread getImageThread() throws CorruptDataException,
			DataUnavailable {		
			throw new DataUnavailable();
	}
	

	public void addStackFrame(JStackFrame frame) {
		frames.add(frame);
	}
	
	@Override
	public ImagePointer getJNIEnv() throws CorruptDataException {
		
		return new SimpleImagePointer(id);
	}
	@Override
	public String getName() throws CorruptDataException {
		
		return threadName;
	}
	@Override
	public JavaObject getObject() throws CorruptDataException {
		return object;
	}
	
	public void setObject(JObject object) {
		this.object = object;
	}
	
	@Override
	public int getPriority() throws CorruptDataException {
		return priority;
	}
	
	@Override
	public List<JavaStackFrame> getStackFrames() {
		CLogger.logr.log(Level.FINEST,"Asked for stack frames");
		return frames;
	}
	
	@Override
	public List<ImageSection> getStackSections() {
		return Collections.emptyList();
	}
	
	@Override
	public int getState() throws CorruptDataException {
		return state;
	}
}