/*******************************************************************************
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 *   http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 ******************************************************************************/

package org.apache.kato.jvmti.javaruntime.model;

import java.util.Collections;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import javax.tools.diagnostics.runtime.java.JavaClass;
import javax.tools.diagnostics.runtime.java.JavaClassLoader;
import javax.tools.diagnostics.runtime.java.JavaHeap;
import javax.tools.diagnostics.runtime.java.JavaMethod;
import javax.tools.diagnostics.runtime.java.JavaMonitor;
import javax.tools.diagnostics.runtime.java.JavaObject;
import javax.tools.diagnostics.runtime.java.JavaThread;

public class Model {

	private ObjectMapList<Long,JavaThread> threadMap=new ObjectMapList<Long,JavaThread>();
	Map<Long,JavaClass>  classMap=new HashMap<Long,JavaClass>();
	private ObjectMapList<Long,JavaClassLoader> loadersMap=new ObjectMapList<Long,JavaClassLoader>();
	private ObjectMapList<Long,JavaMonitor> monitorsMap=new ObjectMapList<Long,JavaMonitor>();
	private ObjectMapList<Long,JavaMethod>  methodMap=new ObjectMapList<Long,JavaMethod>();
	private ObjectMapList<Long,JavaObject>  objectMap=new ObjectMapList<Long,JavaObject>();

	// Exists so that artificial classes can be added before actual system loaders is read. 
	private JClassLoader systemClassLoader = new JClassLoader();
	
	public JClass getClass(long id) {
		if(id==0) return null;
		JClass m=(JClass) classMap.get(id);
		if(m==null) {
			m=new JClass(this);
			m.classid=id;
			classMap.put(id,m);
		}
		return m;
	}

	public JavaObject getObjectAtAddress(long loc){
		return objectMap.get(loc);
	}


	public JMethod getMethod(long methodid) {
		if(methodid==0) return null;
		JMethod m=(JMethod) methodMap.get(methodid);
		if(m==null) {
			m=new JMethod();

			methodMap.put(methodid,m);
		}
		return m;

	}
	public JClassLoader getLoader(long classloaderid) {
		JClassLoader loader=(JClassLoader) loadersMap.get(classloaderid);
		
		return loader;
	}


	public JObject getObject(long classid, long objectid) {

		JObject o=(JObject) objectMap.get(objectid);
		if(o==null) {

			JClass clazz = null;

			// TODO this is potentially wrong
			if (classid !=0 ){
				clazz=getClass(classid);}
			o=new JObject(clazz,objectid, this);
			objectMap.put(objectid,o);
		}
		return o;
	}
	
	public void putObject(long objectid, JObject obj) {
		if ((objectid != 0L) && (obj != null)) {
			objectMap.put(objectid, obj);
		}
	}
	
	public JMonitor getMonitor(long monitorid) {
		if(monitorid==0) return null;
		JMonitor monitor=(JMonitor) monitorsMap.get(monitorid);
		if(monitor==null) {
			monitor=new JMonitor(monitorid, this);
			monitorsMap.put(monitorid,monitor);
		}
		return monitor;
	}
	public List<JavaThread> getThreads() {
		return threadMap.values();
	}
	public List<JavaClassLoader> getJavaClassLoader() {
		return loadersMap.values();
	}
	public List<JavaMonitor> getJavaMonitors() {
		return monitorsMap.values();
	}
	
	public JThread getThread(long id) {
		JThread t=(JThread) threadMap.get(id);
		if(t!=null) return t;
		t=new JThread(id);
		threadMap.put(t.id, t);
		return t;
	}

	/**
	 * Returns a list containing all of the JavaObjects.
	 * Makes sure that the list is unmodifiable.
	 *  
	 * @return an unmodifiable list of objects.
	 */
	public List<JavaObject> getObjects() {
		return Collections.unmodifiableList(objectMap.values());		
	}

	private JHeap heap;
	
	/**
	 * Returns the JavaHeap.
	 * 
	 * @return JHeap instance
	 */
	public JavaHeap getHeap() {
		if (heap == null) {
			heap = new JHeap(this);
		}
		
		return heap;
	}

	/**
	 * Store a new classloader.
	 * 
	 * @param position its position in the file
	 * @param classLoader Its object
	 */
	public void putLoader(long position, JavaClassLoader classLoader) {
		this.loadersMap.put(position, classLoader);
	}

	public JClassLoader getSystemClassLoader() {
		return systemClassLoader;
	}
}
