/*******************************************************************************
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 *   http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 ******************************************************************************/
package org.apache.kato.jvmti.process;

import javax.tools.diagnostics.image.CorruptData;
import javax.tools.diagnostics.image.CorruptDataException;
import javax.tools.diagnostics.image.ImagePointer;

/**
 * Implementation of CorruptData.
 * Very simply - all implementations should be like this.
 * 
 */
public class CorruptDataImpl implements CorruptData {
	private String string;
	private ImagePointer pointer = null;
	
	/**
	 * Convenience method for creating CorruptDataException instances.
	 * They always take a CorruptData object.
	 * 
	 * @param desc Description of corrupt data.
	 * @return a CorruptDataException with an initialized CorruptData instance.
	 */
	static public CorruptDataException corruptDataException(String desc) {
		return new CorruptDataException(new CorruptDataImpl(desc));
	}
	
	/**
	 * Convenience method for creating CorruptDataException instances.
	 * They always take a CorruptData object.
	 * 
	 * @param desc Description of corrupt data.
	 * @param pointer locations of corrupt data.
	 * @return a CorruptDataException with an initialized CorruptData instance.
	 */
	static public CorruptDataException corruptDataException(String desc, ImagePointer pointer) {
		return new CorruptDataException(new CorruptDataImpl(desc, pointer));
	}
	
	/**
	 * Constructor for CorruptDataImpl. Takes a String. 
	 * @param desc description of the corrupt data.
	 */
	public CorruptDataImpl(String desc) {
		this.string = desc;
	}
	
	/**
	 * Constructor for CorruptDataImpl. Takes a String. 
	 * @param desc description of the corrupt data.
	 * @param pointer ImagePointer to locate corrupt data. 
	 */	
	public CorruptDataImpl(String desc, ImagePointer pointer) {
		this.string = desc;
		this.pointer = pointer;
	}
	
	@Override
	public ImagePointer getAddress() {
		return pointer;
	}

	@Override
	public String toString() {
		return string +
		((pointer == null) ? "" : ", @  0x" + Long.toHexString(pointer.getAddress()));	
	}
}
