/*******************************************************************************
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 *   http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 ******************************************************************************/
package org.apache.kato.common142;

import java.io.File;

/**
 * This class provides access to native function used to send a signal back to the 
 * invoking JVM with the intent of triggering a Dump.
 * 
 */
public class DumpTrigger {

	private static final String libraryPathName="org.apache.kato.dumptrigger.librarypath";
	
	private static DumpTrigger trigger=null;
	
	static {
		new DumpTrigger();
	}
	 
	
	public DumpTrigger () {
		
		String pathName=System.getProperty(libraryPathName);
		if(pathName!=null) {
			pathName=pathName.trim();
			if(pathName.equals("")) throw new RuntimeException("DumpTrigger pathname system property "+libraryPathName+" is empty");
			System.load(pathName);
			trigger=this;
		}
		else {
			// no overt specification of a library so lets go looking...
			// look for a file with katovmapi in the name...
			pathName=findLocalLibrary();
			if(pathName!=null) {
				System.load(pathName);
				trigger=this;
			}
		}
		
		
		
	}
	
	private String findLocalLibrary() {
		
		
		String lib=findLib(".");
		if(lib!=null) return lib;
		lib=findLib("lib");
		
		return lib;
		
	}

	private String findLib(String location) {
		File dir=new File(location);
		if(dir.exists()==false) return null; // doesn't exists
		if(dir.isDirectory()==false) return null; // not a directory
		File[] kids=dir.listFiles();
		if(kids==null || kids.length==0) return null; // no contents
		
		for(int i=0;i<kids.length;i++) {
			File child=kids[i];
			if(child.isFile()) {
				String name=child.getName();
				if(name.indexOf("katovmapi")>=0) return child.getAbsolutePath();
			}
		}
		return null;
	}

	public  static boolean triggerDump() {
		
		if(trigger==null) return false;
		
		int pid=getProcessID();
		return trigger.sendDumpSignal(pid);
	}

	public static int getProcessID() {
		if(trigger==null) return 0;
		return trigger.getNativeProcessID();
	}

	private native int getNativeProcessID();
	private native boolean sendDumpSignal(int procid);
	
}
