/*******************************************************************************
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 *   http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 ******************************************************************************/
package test.apache.kato.hprof.image;


import java.io.IOException;
import java.util.Iterator;
import java.util.List;

import javax.tools.diagnostics.image.CorruptDataException;
import javax.tools.diagnostics.image.DataUnavailable;
import javax.tools.diagnostics.image.Image;
import javax.tools.diagnostics.image.ImageAddressSpace;

import org.apache.kato.hprof.image.ImageImpl;


public class TestImage extends AbstractHProfTestCase {

	
	public void testNullConstructor() {
		
		try {
		new ImageImpl(null);
		fail("expected illegal argument exception");
		} catch(IllegalArgumentException iae) {
			;
		}
		
	}
	
	
	public void testGetCreationTime() throws IOException, DataUnavailable {
		
			Image minimal=getMinimalImage();
			long creationTime=minimal.getCreationTime();
			assertEquals("creation time is incorrect",0xF0F0,creationTime);
			
		
	}
	
	public void testGetHostName() throws IOException, CorruptDataException {
		try {
			Image minimal=getMinimalImage();
			minimal.getHostName();
			fail("expected DataUnavailableException");
		} catch(DataUnavailable du) {
			
		}
	}
	
	public void testGetIPAddresses() throws IOException, CorruptDataException {
		try {
			Image minimal=getMinimalImage();
			minimal.getIPAddresses();
			fail("expected DataUnavailableException");
		} catch(DataUnavailable du) {
			
		}
	}
	public void testGetInstalledMemory() throws IOException, CorruptDataException {
		try {
			Image minimal=getMinimalImage();
			minimal.getInstalledMemory();
			fail("expected DataUnavailableException");
		} catch(DataUnavailable du) {
			
		}
	}
	public void testGetProcessorCount() throws IOException, CorruptDataException {
		try {
			Image minimal=getMinimalImage();
			minimal.getProcessorCount();
			fail("expected DataUnavailableException");
		} catch(DataUnavailable du) {
			
		}
	}
	public void testGetProcessorSubType() throws IOException, CorruptDataException {
		try {
			Image minimal=getMinimalImage();
			minimal.getProcessorSubType();
			fail("expected DataUnavailableException");
		} catch(DataUnavailable du) {
			
		}
	}
	public void testGetProcessorType() throws IOException, CorruptDataException {
		try {
			Image minimal=getMinimalImage();
			minimal.getProcessorType();
			fail("expected DataUnavailableException");
		} catch(DataUnavailable du) {
			
		}
	}
	public void testGetSystemType() throws IOException, CorruptDataException {
		try {
			Image minimal=getMinimalImage();
			minimal.getSystemType();
			fail("expected DataUnavailableException");
		} catch(DataUnavailable du) {
			
		}
	}
	public void testGetSystemSubType() throws IOException, CorruptDataException {
		try {
			Image minimal=getMinimalImage();
			minimal.getSystemSubType();
			fail("expected DataUnavailableException");
		} catch(DataUnavailable du) {
			
		}
	}
	
	
	public void testGetAddressSpaceIterator() throws IOException {
		
			Image minimal=getMinimalImage();
			List i=minimal.getAddressSpaces();
			assertNotNull(i);
	}

	public void testGetNonEmptyAddressSpaceIterator() throws IOException {
		
		Image minimal=getMinimalImage();
		Iterator i=minimal.getAddressSpaces().iterator();
		assertTrue(i.hasNext());
	}

	public void testGetRealAddressSpace() throws IOException {
		
		Image minimal=getMinimalImage();
		Iterator i=minimal.getAddressSpaces().iterator();
		Object o=i.next();
		assertTrue("object is not an image address space",o instanceof ImageAddressSpace);
		
	}
	
}
