/*******************************************************************************
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 *   http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 ******************************************************************************/
package test.apache.kato.hprof.java;

import java.io.File;
import java.io.IOException;
import java.util.Iterator;
import java.util.List;

import javax.tools.diagnostics.image.CorruptDataException;
import javax.tools.diagnostics.image.DataUnavailable;
import javax.tools.diagnostics.image.ImagePointer;
import javax.tools.diagnostics.image.MemoryAccessException;
import javax.tools.diagnostics.runtime.java.JavaObject;
import javax.tools.diagnostics.runtime.java.JavaRuntime;
import javax.tools.diagnostics.runtime.java.JavaVMInitArgs;


import test.apache.kato.hprof.image.AbstractHProfTestCase;

public class TestJavaRuntime extends AbstractHProfTestCase{

	public void testGetFullVersion() throws IOException, CorruptDataException {

		JavaRuntime runtime=getJavaRuntime();
		assertNotNull(runtime.getFullVersion());

	}
	public void testGetVersion() throws IOException, CorruptDataException {

		JavaRuntime runtime=getJavaRuntime();
		assertNotNull(runtime.getVersion());

	}


	public void testEqualsPasses() throws IOException {
		JavaRuntime runtime=getJavaRuntime();
		assertTrue(runtime.equals(runtime));

	}
	public void testEqualsFails() throws IOException {
		JavaRuntime runtime=getJavaRuntime();
		assertFalse(runtime.equals(new MockJavaRuntime()));

	}

	public void testGetMonitors() throws IOException {

		JavaRuntime runtime=getJavaRuntime();
		List l=runtime.getMonitors();

		int count=l.size();
		
		assertEquals("More than 0 monitors returned. There are none.",0,count);
	}

	public void testGetJVM() throws IOException, DataUnavailable, CorruptDataException {
		JavaRuntime runtime=getJavaRuntime();

		
		ImagePointer pointer=(ImagePointer) runtime.getJavaVM();
		assertNotNull(pointer);

	}

	public void testGetVMArguments() throws CorruptDataException, IOException {
		JavaRuntime runtime=getJavaRuntime();
		try {
			runtime.getJavaVMInitArgs();
			fail("expected DataUnavailable exception");
		} catch(DataUnavailable du) {

		}

	}

	public void testGetCompiledMethodsReturns() throws IOException {
		JavaRuntime runtime=getJavaRuntime();
		List i=runtime.getCompiledMethods();
		assertNotNull(i);

	}
	public void testGetCompiledMethodsIteratorIsEmpty() throws IOException {
		JavaRuntime runtime=getJavaRuntime();
		Iterator i=runtime.getCompiledMethods().iterator();
		assertFalse("compiled methods iterator is not empty",i.hasNext());

	}

	public void testGetHeapIteratorReturns() throws IOException {
		JavaRuntime runtime=getPopulatedJavaRuntime();
		List i=runtime.getHeaps();
		assertNotNull(i);

	}
	private final class MockJavaRuntime implements JavaRuntime {
		@Override
		public List getCompiledMethods() {

			return null;
		}

		@Override
		public List getHeapRoots() {

			return null;
		}

		@Override
		public List getHeaps() {

			return null;
		}

		@Override
		public List getJavaClassLoaders() {

			return null;
		}

		@Override
		public ImagePointer getJavaVM() throws CorruptDataException {

			return null;
		}

		@Override
		public JavaVMInitArgs getJavaVMInitArgs() throws DataUnavailable,
		CorruptDataException {

			return null;
		}

		@Override
		public List getMonitors() {

			return null;
		}

		@Override
		public JavaObject getObjectAtAddress(ImagePointer address)
		throws CorruptDataException, IllegalArgumentException,
		MemoryAccessException, DataUnavailable {

			return null;
		}

		@Override
		public List getThreads() {

			return null;
		}

		@Override
		public Object getTraceBuffer(String bufferName, boolean formatted)
		throws CorruptDataException {

			return null;
		}

		@Override
		public String getFullVersion() throws CorruptDataException {

			return null;
		}

		@Override
		public String getVersion() throws CorruptDataException {

			return null;
		}

		@Override
		public File getSource() {
			// TODO Auto-generated method stub
			return null;
		}
	}
}
