/*******************************************************************************
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 *   http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 ******************************************************************************/
package org.apache.kato.hprof;

import org.apache.kato.hprof.datalayer.ConstantPoolEntry;
import org.apache.kato.hprof.datalayer.InstanceFieldEntry;
import org.apache.kato.hprof.datalayer.StaticFieldEntry;

/**
 * Allows JavaClass information to be pulled from HProf records.
 * 
 *
 */
public interface IJavaClass {
	/**
	 * @return
	 */
	public long getClassObjectID();

	public int getStackTraceSerialNumber();

	public String getClassName();

	public int getClassSerialNumber();
	public long getClassLoaderObjectID();
	public long getProtectionDomainObjectID();
	public long getSignersObjectID();
	
	/**
	 * Get the object ID of the superclass.
	 * 
	 * @return long object id.
	 */
	public long getSuperClassObjectID();

	public int getInstanceSize();

	public ConstantPoolEntry[] getConstantPoolEntries();
	
	public StaticFieldEntry[] getStaticFields();
	
	public InstanceFieldEntry[] getInstanceFields();

	public int getOffsetSize();
}
