/*******************************************************************************
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 *   http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 ******************************************************************************/
package org.apache.kato.hprof.datalayer;

/**
 * Represents a static field in a class on the heap.
 * Provides methods to retrieve its value.
 */
public class StaticFieldEntry extends FieldEntry {
	public StaticFieldEntry(long fieldNameID, short type, long data) {
		this.fieldNameID = fieldNameID;
		this.type = type;
		this.data = data;
	}
	
	/**
	 * The data. Fields are either ID's of objects or numbers,
	 * all should fit within a long. 
	 */
	private long data;
	
	public long getData() {
		return data;
	}
	
	public byte getByteField() {
		return (byte) data;
	}


	public char getCharField() {
		return (char) data;
	}


	public double getDoubleField() {
		return Double.longBitsToDouble(data);
	}


	public float getFloatField() {
		return Float.intBitsToFloat((int) data);
	}


	public long getIDField() {		
		return data;
	}


	public int getIntegerField() {
		return (int)data;
	}

	
	public long getLongField() {	
		return data;
	}


	public short getShortField() {
		return (short)data;
	}

	public boolean getBooleanField() {
		return !(data == 0L);
	}
}