/*******************************************************************************
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 *   http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 ******************************************************************************/
package org.apache.kato.tools.jdi;
import java.io.IOException;
import java.net.SocketAddress;
import java.nio.ByteBuffer;
import java.nio.channels.ClosedSelectorException;
import java.nio.channels.SelectionKey;
import java.nio.channels.Selector;
import java.nio.channels.SocketChannel;
import java.util.Iterator;

/* JDISocket
 * 
 * JDISocket is the first and last point at which the client's commands go through
 * the various layers.  We read on the client, and write to writeRawPacket() in
 * JDWPServer.
 */

public class JDISocket extends Thread{
	private JDISocketServer vsockserve;
	private JDWPServer jdwpServer;
	private SocketChannel chan;
	private boolean disconnecting = false;
	private JDILogger logr = JDILogger.logger;
	
	public JDISocket(){
		
	}
	
	public JDISocket(JDWPServer jdwp, SocketChannel inChan, JDISocketServer vsockserve){
		jdwpServer = jdwp;
		chan = inChan;
		SocketAddress remote = inChan.socket().getRemoteSocketAddress();
		logr.log(JDILogger.LEVEL_VERBOSE, "Connected to" + remote.toString()); //$NON-NLS-1$
		this.vsockserve = vsockserve;
		this.start();
	}

	public void write(byte []bytes){
		if (chan.isConnected()){
			try{
				//Lets take up some more memory to do something simple
				ByteBuffer bbuf = ByteBuffer.wrap(bytes);
				chan.write(bbuf);
			}catch(Exception exxy){
				System.err.println("Error: " + exxy.getMessage()); //$NON-NLS-1$
				exxy.printStackTrace();
			}
		}
	}

	public void disconnect(){
		disconnecting = true;
		try{
			chan.close();
			selector.close();
			logr.log(JDILogger.LEVEL_VERYVERBOSE, "Shutting down connection"); //$NON-NLS-1$
		}catch(IOException exxy){ 
			logr.logError(JDILogger.LEVEL_NORMAL, "Error disconnecting: " + exxy.getLocalizedMessage()); //$NON-NLS-1$
			exxy.printStackTrace();
		}

	}

	private Selector selector;
	
	@Override
	public void run(){
		try{
			selector = Selector.open();
			if (chan == null || !chan.isConnected()){
				logr.logError(JDILogger.LEVEL_QUIET, "Unable to use SocketChannel"); //$NON-NLS-1$
			}

			chan.configureBlocking(false);
			chan.register(selector, SelectionKey.OP_READ);

			while(true){
				selector.select(250);
				Iterator<SelectionKey> itr = selector.selectedKeys().iterator();
				while (itr.hasNext()){
					SelectionKey key = itr.next();
					if (key.isValid()){
						//Also, lets make this overly simple operation really bad
						SocketChannel sc = (SocketChannel)key.channel();
						ByteBuffer bbuf = ByteBuffer.allocate(1024);
						int rd = sc.read(bbuf);
						if (rd > 0){
							bbuf.flip();
							byte []readb = new byte[rd];
							bbuf.get(readb);
							jdwpServer.writeRawPacket(readb, this);
							
						}

					}
				}
			}

		}
		catch(IOException exxy){
			try{
				if (!disconnecting)
				chan.close();
			}catch(Exception exxy2){ }
			chan = null;
		}
		catch(ClosedSelectorException exxy){
			try{
				if (!disconnecting)
				chan.close();
			}catch(Exception exxy2){ }
			chan = null;
		}
		catch(Exception exxy){
			try{
				chan.close();
			}catch(Exception exxy2){ }
			System.err.println("Exception: " + exxy.getLocalizedMessage()); //$NON-NLS-1$
			exxy.printStackTrace();
			chan = null;
			jdwpServer.stop();
			
		}
		vsockserve.notifyDisconnect();

	}

	public boolean isConnected(){
		if (chan == null){
			return false;
		}
		return chan.isConnected();
	}




}
