/*******************************************************************************
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *   http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 ******************************************************************************/
#include <stdio.h>
#include <stdlib.h>
#include <malloc.h>
#include <string.h>

// This is really an array list.

struct queueADT{
	void * queue;
	int readPoint;
	int addPoint;
	int size;
	int sizeOfElement;
	int qID;
};

static int id;
int doubleSize(queueADT);

// Call this to initialise the array list
struct queueADT * Q_initQueue(int qsize, int sizeofElement){
	void * pt;
	struct queueADT * q = calloc(1, sizeof(struct queueADT));
	id++;
	q->qID = id;
	q->sizeOfElement = sizeofElement;
	q->size = qsize;
	q->addPoint = 0;
	q->readPoint = 0;
	pt = calloc (qsize, sizeofElement);
	if (pt == NULL){
		printf("Problem with malloc \n");
		abort();
	}
	q->queue = pt;
	return q;
}

int Q_dequeue (void * writeLoc, struct queueADT * q){
	char * src;
	if (q == NULL){
		printf("null queue passed, abort \n");
	}
	if (q->readPoint == q->addPoint){
		memset(writeLoc, 0, q->sizeOfElement);
		return 0;
	}
	src = (char *)q->queue;
	memcpy(writeLoc, (void *)&src[q->readPoint*q->sizeOfElement], q->sizeOfElement);
	q->readPoint++;
	q->readPoint = (q->readPoint)%q->size;
	return 1;
}

int Q_enqueue(void * readLoc, struct queueADT * q){
	char * dst;
	if (q == NULL){
		printf("null queue passed, abort \n");
		abort();
	}
	if (((q->addPoint+1)%q->size) == q->readPoint){
		doubleSize(q);
	}
	dst = q->queue;
	memcpy((void *)&dst[q->addPoint*q->sizeOfElement], readLoc, q->sizeOfElement);
	q->addPoint = (q->addPoint+1)%(q->size);
	return 1;
}

int Q_size(struct queueADT *q){
	return q->size;
}


// Expands the size of the queue (array list), copies values and deallocs the old array 
int doubleSize(struct queueADT * q){
	void * pt;
	int i;
	char * src;
	char * dst;
	if (q == NULL){
		printf("null queue passed, abort \n");
	}
	pt = calloc(q->size*2, q->sizeOfElement);
	if (pt == NULL){
		printf("Problem with malloc \n");
		abort();
	}
	src = q->queue;
	dst = pt;
	for (i = 0; i < q->size; i++){
		if (i == 0 && q->readPoint == 0){
			memcpy((void *)&dst[i*(q->sizeOfElement)], (void *)&src[0], q->sizeOfElement);
		}else{
			memcpy((void *)&dst[i*(q->sizeOfElement)], (void *)&src[(((q->readPoint)+i)%(q->size))*(q->sizeOfElement)], q->sizeOfElement);
		}
	}
	if (q == q->queue){
		printf( "? ??? \n ");
	}
	free(q->queue);
	printf("%d doubled %d \n",q->qID, q->size*2);
	if (q->addPoint >= q->readPoint){
		q->addPoint = q->addPoint - q->readPoint;
	}else{
		q->addPoint = q->size - q->readPoint + q->addPoint;
	}
	q->readPoint = 0;
	q->queue = pt;
	q->size = q->size*2;
	return 1;
}


int Q_free(struct queueADT * q){
	id--;
	free(q->queue);
	free(q);
	q = NULL;
	return 1;
}
