/*******************************************************************************
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 *   http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 ******************************************************************************/
package test.apache.kato.vmapi;

import java.io.File;

import junit.framework.TestCase;

import org.apache.kato.common142.DumpTrigger;

public class TestLibraryLoad extends TestCase {

	private static String getLibName() {
		Object libloc=System.getProperties().get("libloc");
		String location=null;
		
		if(libloc==null) {
			location="../target";
		}
		else  {
			//if(libloc==null) throw new RuntimeException("system property libloc which defines the directory containing the native lib is missing");
		 location=libloc.toString();
		}
	
		File dir=new File(location);
		if(dir.exists()==false) {
			 throw new RuntimeException("Directory location defined by system property libloc does not exist [libloc="+location+"]"); 
		}
		if(dir.isDirectory()==false) {
			 throw new RuntimeException("Directory location defined by system property libloc exists but is not a directory [libloc="+location+"]"); 
		}
		
	
		for(File kid:dir.listFiles()) {
			String name=kid.getName();
			// we expect lib or dlls
			
			if(name.startsWith("libkatovmapi")) return kid.getAbsolutePath();
			if(name.startsWith("katovmapi") && name.endsWith(".dll")) return kid.getAbsolutePath();
			
		}
		 throw new RuntimeException("system property libloc location does not hold a file matching *katovmapi* [libloc="+location+"]");
		
	}
	
	public void testTrigger() {
		
		loadLibrary();
		boolean result=DumpTrigger.triggerDump();
		assertTrue(result);
	}
	public void testGetProcessID() {
		
		loadLibrary();
		int id=DumpTrigger.getProcessID();
		assertFalse("id is zero",id==0);
	}

	private void loadLibrary() {
		String libname=getLibName();	
		System.load(libname);
	}
}
