/*******************************************************************************
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 *   http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 ******************************************************************************/
package org.apache.kato.tck.harness;


public class BuildVersionHelper {

	
	private static final String JAVA_SPECIFICATION_VERSION = "java.specification.version";
	private static final String JAVA_VENDOR = "java.vendor";
	private static final String JAVA_RUNTIME_VERSION = "java.runtime.version";

	/**
	 * 
	 * @return jvm build version in the form  version.subversion.subversion.subversion... 
	 */
	
	public static String getBuildVersion() {
		
	
		
		String version=getProperty(JAVA_SPECIFICATION_VERSION);
		String vendor=getProperty(JAVA_VENDOR).toLowerCase();
		
		
		if(vendor.indexOf("sun")>=0) {
			
			return getSunBuildVersion(getProperty("java.runtime.version"));
		}
		else if(vendor.indexOf("ibm")>=0) {
			if(version.equals("1.4")) {
				
				return getIBM142Version(getProperty("java.fullversion"));	
			}
			if(version.equals("1.5")) {
				
				return getIBM150Version(getProperty("java.runtime.version"));	
			}
			if(version.equals("1.6")) {
				
				return getIBM160Version(getProperty("java.runtime.version"));	
			}
			else throw new IllegalArgumentException("unable to handle IBM Java version "+version);
			
		}
		
		for(int i=2000;i<2050;i++) {
			String id=""+i;
			int loc=version.indexOf(id);
			if(loc<0) {
				continue;
			}
			else {
			String year=version.substring(loc,loc+4);
			String month=version.substring(loc+4,loc+6);
			String day=version.substring(loc+6,loc+8);
			return year+"/"+month+"/"+day;
				
			}
		}
		
		return null;
	}
	
	
	public static String getIBM160Version(String fullversion) {
		int eyecatcher=fullversion.indexOf("-");
		if(eyecatcher<0) throw new IllegalArgumentException("cannot locate IBM 1.6.0 version info from "+fullversion);
		
		int us=fullversion.indexOf("_",eyecatcher);
		
		String datestamp=fullversion.substring(eyecatcher+1,us);
		String sr=fullversion.substring(us+1);
		
		// get service refresh number if available
		
		
		String result="1.6.0."+sr+"."+datestamp;
		
		
		return result;
		
	}
	public static String getIBM150Version(String fullversion) {
		int eyecatcher=fullversion.indexOf("-");
		if(eyecatcher<0) throw new IllegalArgumentException("cannot locate IBM 1.5.0 version info from "+fullversion);
		
		// now expect date + month + day etc
		int space=fullversion.indexOf(' ', eyecatcher);
		String datestamp=fullversion.substring(eyecatcher+1,space);
		
		// get service refresh number if available
		
		String sr=null;
		
		// find first open bracket
		
		int ob=fullversion.indexOf("(",space);
		if(ob>eyecatcher) {
			int cb=fullversion.indexOf(")",space);
			sr=fullversion.substring(ob+1,cb);
		}
		
		String result="1.5.0";
		if(sr!=null) result=result+"."+sr+"."+datestamp;
		else         result=result+".0."+datestamp;
		
		
		return result;
	}
	public static String getIBM142Version(String fullversion) {
		
		int eyecatcher=fullversion.indexOf("-");
		if(eyecatcher<0) throw new IllegalArgumentException("cannot locate IBM 1.4.2 version info from "+fullversion);
		
		// now expect date + month + day etc
		int space=fullversion.indexOf(' ', eyecatcher);
		String datestamp=fullversion.substring(eyecatcher+1,space);
		
		// get service refresh number if available
		
		String sr=null;
		
		// find first open bracket
		
		int ob=fullversion.indexOf("(",space);
		if(ob>eyecatcher) {
			int cb=fullversion.indexOf(")",space);
			sr=fullversion.substring(ob+1,cb);
		}
		
		String result="1.4.2";
		if(sr!=null) result=result+"."+sr+"."+datestamp;
		else         result=result+".0."+datestamp;
		
		
		return result;
	}

	private static String getProperty(String id) {
		String result=System.getProperty(id);
		if(result==null) throw new IllegalArgumentException("cannot location required system property "+id);
		return result;
	}

	

	public static String getSunBuildVersion(String version) {
		char chars[]=version.toCharArray();
		
		for(int i=0;i<chars.length;i++) {
			char c=chars[i];
			
			if(Character.isDigit(c)) continue;
			if(c=='_' || c=='-' || c=='.') {
				chars[i]='.';
				continue;
			}
			chars[i]=0;
		}
		
		StringBuffer results=new StringBuffer();
		char last=0;
		for(int i=0;i<chars.length;i++) {
			char c=chars[i];
			if(c==0) continue; // skip null char
			
			if(Character.isDigit(c)) {
				results.append(c);
				last=c;
			}
			else {
				if(last=='.') continue; // duplicate dots - skip
				results.append(c);
				last=c;
			}
		}
		
		return new String(results);
	}
}
