/*******************************************************************************
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 *   http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 ******************************************************************************/
package org.apache.kato.tck.harness;

import java.lang.reflect.Method;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.List;
import java.util.Map;
import java.util.TreeMap;

import org.apache.kato.tck.api.TCKException;


public class ValueCollector {

	private Map variables=new TreeMap();

	/**
	 * Creates collector with default values
	 * @param arguments
	 */
	public ValueCollector(List arguments) {
		
		
		Iterator i=arguments.iterator();
		while(i.hasNext()) {
			String value=(String) i.next();
			collect(value,null);
		}
	}
	public ValueCollector() {

	}


	/**
	 * Collects  variable settings
	 * 
	 * If a method registers a conflicting requirement
	 * a TCK exception is thrown.
	 *  
	 * @param var - variable to record 
	 * @param method - method doing the recording
	 * 
	 */

	public void collect(String key,Method method) {

		if(variables.containsKey(key)==false) {
			variables.put(key, method);

		}
		else {
			// key clash
			Method original=(Method) variables.get(key);
			throw new TCKException("unable to register key "+key+" for "+method+" value conflicts with previously registered key from "+original);
		}
	}

	/**
	 * Returns values as a list
	 * 
	 */
	public List asList() {
		List list=new LinkedList();
		list.addAll(variables.keySet());
		return list;
	}
	/**
	 * @return
	 */
	public int size() {
		return variables.size();
	}
}
