/*******************************************************************************
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 *   http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 ******************************************************************************/
package org.apache.kato.tck.results.reporter;



import java.util.Iterator;
import java.util.LinkedList;
import java.util.List;
import java.util.Map;
import java.util.TreeMap;
import java.util.TreeSet;

public class TCKResultMatrix {
	
	private TCKGroup root=new TCKGroup();
	private int passes=0;
	
	private SortKeyProvider keyProvider=new VersionSortKeyProvider();
	
	private TreeSet testnames=new TreeSet();

	public TCKResultMatrix(SortKeyProvider sortKeyProvider) {
		keyProvider=sortKeyProvider;
	}
	public TCKResultMatrix() {
		
	}
	
	
	public Iterator getTCKResultSets() {
		List data=new LinkedList();
		root.addResultSets(data);
		return data.iterator();
		
	}
	public void addResultSet(TCKResultSet resultSet) {
		
		// 	add testnames to matrix lefthand side
		
		Iterator ri=resultSet.iterator();
		while(ri.hasNext()) {
		TestcaseResult result=(TestcaseResult) ri.next();
			
			
			testnames.add(result.getLabel());
			if(result instanceof TestcaseResultPass) passes++;
		}
		
		// store results at right level in tree
		
		String key=keyProvider.getkey(resultSet.getConfig());
		TCKGroup versionGroup=root.makeChildGroup(key);
		versionGroup.addResults(resultSet);	
	
		
		
	}

	public int columnCount() {
		if(root.isNode()) return 0;
		return root.columnCount();
	}
	public String[] getRowNames() {
	return (String[]) testnames.toArray(new String[0]);
	}

	public boolean hasColumns() {
		
		return root.hasChildren();
	}

	
	
	class TCKGroup {
		private Key key=null;
		
			
		private TreeMap childGroups=new TreeMap();
		private TCKGroup parent=null;
		private int errors=0;
		private int failures=0;
		private Map data=new TreeMap();
		private List results=new LinkedList();
		
		public TCKGroup(TCKGroup parent,String name) {
			this.parent=parent;
			if(parent!=null) {
				Key pKey=parent.getKey();
				key=new Key(pKey,name);
			}
			else this.key=new Key(name);
		}
		
		public boolean isNode() {
			return childGroups.isEmpty();
		}
		public void addResultSets(List list) {
			
			list.addAll(results);
			Iterator i=childGroups.values().iterator();
			while(i.hasNext()) {
				TCKGroup kids=(TCKGroup) i.next();
				kids.addResultSets(list);
			}
			
			
		}


		public TCKGroup() {
		
		}

		public String toString() {
			return getKey().toString();
		}
		public int columnCount() {
			
			if(childGroups.isEmpty()) {
				return 1;
			}
			else {
				int count=0;
				Iterator i=childGroups.values().iterator();
				while(i.hasNext()) {
					TCKGroup kids=(TCKGroup) i.next();
					count+=kids.columnCount();
				}
				
				return count;
			}
			
		}
		public boolean hasChildren() {
			return childGroups.isEmpty()==false;
		}
		
		public void addResults(TCKResultSet resultSet) {
			
			results.add(resultSet);
			
			Iterator ri=resultSet.iterator();
			while(ri.hasNext()) {
			TestcaseResult r=(TestcaseResult) ri.next();
			
				data.put(r.getLabel(),r);
				if(r instanceof TestcaseResultFailure) failures++;
				if(r instanceof TestcaseResultError) errors++;
			}
			

		}
		
		public Key getKey() {
			return key;
			
			
		}
		public TCKGroup makeChildGroup(String  keyPath) {
			if(keyPath==null) keyPath=""; 
			String[] key=keyPath.split("/");
			TCKGroup top=this;
			for(int k=0;k<key.length;k++) {
			String keySegment=key[k];
				TCKGroup childGroup=null;
				if(top.childGroups.containsKey(keySegment)==false) {
					childGroup=new TCKGroup(top,keySegment);
					top.childGroups.put(keySegment,childGroup);
				}
				else {
					childGroup=(TCKGroup) top.childGroups.get(keySegment);
				}
				top=childGroup;
			}
			
			return top;
		}
		public TCKGroup getChildGroup(String name) {
			
			return (TCKGroup) childGroups.get(name);
		}
		public String[] getkeyValues(Key key) {
			
			TCKGroup group=getNode(key);
			return (String[]) group.childGroups.keySet().toArray(new String[0]);
		}
		
		public TCKGroup getNode(Key k) {
			
			if(k==key) return this;
			
			String[] keyParts=k.parts();
			
			TCKGroup top=this;
			for(int i=0;i<keyParts.length;i++) {
				String keySegment=keyParts[i];
				if(top.childGroups.containsKey(keySegment)) {
					top=(TCKGroup) top.childGroups.get(keySegment);
				}
				else {
					return null;
				}
				
			}
			
			return top;
		}
		public int getKeyColumnWidth(Key keyPath) {
			
			
			TCKGroup group=getNode(keyPath);
			return group.countAllNodeChildren();
		}
		private int countAllNodeChildren() {
			
			
			
			if(childGroups.isEmpty()) {
				return 1;
			}
			else {
				int count=0;
				Iterator gi=childGroups.values().iterator();
				while(gi.hasNext()) {
				TCKGroup group= (TCKGroup) gi.next();
					count+=group.countAllNodeChildren();
				}
				return count;
			}
			
		}
		public void addKeys(List keys, int depth) {
			
			if(depth==0) {
				Iterator gi=childGroups.values().iterator();
				while(gi.hasNext()) {
				TCKGroup group= (TCKGroup) gi.next();
				
					keys.add(group.getKey());
				}
			}
			else {
				Iterator gi=childGroups.values().iterator();
				while(gi.hasNext()) {
				TCKGroup group= (TCKGroup) gi.next();
				
					group.addKeys(keys,depth-1);
				}
			}
			
		}

		public int getTestcaseCount(Key value) {
			TCKGroup group=getNode(value);
			if(group==null) return 0;
			
			return group.data.size();
		
		}

		public int getErrorCount(Key value) {
			TCKGroup group=getNode(value);
			if(group==null) return 0;
			
			return group.errors; 
		}
		public int getFailureCount(Key value) {
			TCKGroup group=getNode(value);
			if(group==null) return 0;
			return group.failures;
			 
		}

		public TestcaseResult getTestcaseResult(Key value, String rowName) {
			
			TCKGroup group=getNode(value);
			if(group==null) return null;
			
			TestcaseResult result=(TestcaseResult) group.data.get(rowName);
			return result;
		}

		public TCKResultSet getResultSet(Key value, String rowName) {
			TCKGroup group=getNode(value);
			if(group==null) return null;
			TestcaseResult result=(TestcaseResult) group.data.get(rowName);
			if(result==null) return null;
			return result.getResultSet();
		}

		

	}



	public int getkeyDepth() {
		return keyProvider.keyDepth();
	}

	public String[] getKeyValues(Key key) {
	
			return root.getkeyValues(key);
	}

	public int getKeyColumnWidth(Key keyPath) {
		
		return root.getKeyColumnWidth(keyPath);
	}

	
	public Key[] getKeys(int depth) {
		
		 
		List keys=new LinkedList();
		root.addKeys(keys,depth); 
		
		return (Key[]) keys.toArray(new Key[0]);
		
	}

	public SortKeyProvider getKeyProvider() {
		return keyProvider;
	}
	
	
	public int getTestcaseFailureCount(Key value) {
		return root.getFailureCount(value);
	}
	public int getTestcaseErrorCount(Key value) {
		return root.getErrorCount(value);
	}
	
	
	public int getTestcaseCount(Key value) {
		return root.getTestcaseCount(value);
	}
public TestcaseResult getTestcaseResult(Key value, String rowName) {
		
		return root.getTestcaseResult(value,rowName);
	}
public TCKResultSet getResultSet(Key value, String rowName) {
	return root.getResultSet(value,rowName);
}
public int getTestcasePassCount() {
	return passes;
}
public int rowCount() {
	return testnames.size();
}
public void addMatrix(TCKResultMatrix matrix) {
	
	Iterator i=matrix.getTCKResultSets();
	while(i.hasNext()) {
		TCKResultSet set=(TCKResultSet) i.next();
		addResultSet(set);
	}
	
}


}
