/*******************************************************************************
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 *   http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 ******************************************************************************/
package org.apache.kato.tck.scenario142.javaruntime;

import org.apache.kato.tck.harness.scenario.ScenarioElement;

public class SetupJavaClass_uninitialized extends ScenarioElement {
	public void configureJavaClass_uninitialized() {
		TestThread thread =new TestThread();
		
		synchronized (this) {			
			thread.start();
			try {
				while(!running) {
					this.wait();
				}
			} catch (InterruptedException e) {
				e.printStackTrace();
			}
		}
		
		try {
			Thread.sleep(100);
		} catch (InterruptedException e) {
			e.printStackTrace();
		}
		
	}
	
	
	volatile boolean running = false;
	/**
	 * A thread that blocks in a class initialiser.
	 * Uses reflection to prevent enthusiastic JVMs
	 * from blocking in the wrong place. 
	 *  
	 */
	public class TestThread extends Thread {
		public void run() {
			try {
				// Notify our creator that the thread was at least started.
				synchronized(SetupJavaClass_uninitialized.this) {
					SetupJavaClass_uninitialized.this.running = true;
					SetupJavaClass_uninitialized.this.notify();					
				}
				// This call loads the class and initialises it.
				Class clazz = Class.forName( this.getClass().getPackage().getName().concat(".BlockedClass"));
			} catch (ClassNotFoundException e) {
				e.printStackTrace();
			}
		}
	}
}
