/*******************************************************************************
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 *   http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 ******************************************************************************/
package org.apache.kato.tck.scenario142.javaruntime;

import javax.tools.diagnostics.runtime.java.JavaField;
import javax.tools.diagnostics.runtime.java.JavaObject;

import org.apache.kato.tck.harness.TCKJavaRuntimeTestcase;
import org.apache.kato.tck.scenario142.javaruntime.SetupJavaField_getByte;


/**
 * Test JavaField.getByte() against various fields and values.
 *
 */
public class TestJavaField_getByte extends TCKJavaRuntimeTestcase {
	
	SetupJavaField_getByte setup=new SetupJavaField_getByte();
	
	static JavaObject thisObject = null;	
	public JavaObject getScenerioReference() {
		if (thisObject == null) {
			thisObject = super.getScenerioReference();
		}	
		return thisObject;		
	}
	
	public void testMaxByteValue() throws Exception {
		JavaObject thisJavaObject = getScenerioReference();
		JavaField maxByteField = getJavaField(thisJavaObject, "maxByte");
		
		byte maxByteValue = maxByteField.getByte(thisJavaObject);
		
		assertEquals(setup.maxByte, maxByteValue);		
	}
	
	public void testMinByteValue() throws Exception {
		JavaObject thisJavaObject = getScenerioReference();
		JavaField minByteField = getJavaField(thisJavaObject, "minByte");
		
		byte minByteValue = minByteField.getByte(thisJavaObject);
		
		assertEquals(setup.minByte, minByteValue);		
	}

	
	public void testStaticMinByteValue() throws Exception {
		JavaObject thisJavaObject = getScenerioReference();
		JavaField minByteField = getJavaField(thisJavaObject, "staticMinByte");
		
		byte minByteValue = minByteField.getByte(null);
		
		assertEquals(SetupJavaField_getByte.staticMinByte, minByteValue);		
	}
	
	public void testZeroByteValue() throws Exception {
		JavaObject thisJavaObject = getScenerioReference();
		JavaField zeroByteField = getJavaField(thisJavaObject, "zeroByte");
		
		byte zeroByteValue = zeroByteField.getByte(thisJavaObject);
		
		assertEquals(setup.zeroByte, zeroByteValue);		
	}
	
	public void testNegByteValue() throws Exception {
		JavaObject thisJavaObject = getScenerioReference();
		JavaField negByteField = getJavaField(thisJavaObject, "negByte");
		
		byte negByteValue = negByteField.getByte(thisJavaObject);
		
		assertEquals(setup.negByte, negByteValue);		
	}	
	
	public void testPosByteValue() throws Exception {
		JavaObject thisJavaObject = getScenerioReference();
		JavaField posByteField = getJavaField(thisJavaObject, "posByte");
		
		byte posByteValue = posByteField.getByte(thisJavaObject);
		
		assertEquals(setup.posByte, posByteValue);		
	}
	
	public void testDoubleValue() throws Exception {
		JavaObject thisJavaObject = getScenerioReference();
		JavaField doubleValueField = getJavaField(thisJavaObject, "doubleValue");

		try{
			byte aDoubleValue = doubleValueField.getByte(thisJavaObject);
			fail("Missing illegal argument exception from JavaField.getByte(doubleValue)");
		} catch (IllegalArgumentException e) {
			// expected
		}	
	}
	
	public void testFloatValue() throws Exception {
		JavaObject thisJavaObject = getScenerioReference();
		JavaField floatValueField = getJavaField(thisJavaObject, "floatValue");

		try{
			byte aFloatValue = floatValueField.getByte(thisJavaObject);
			fail("Missing illegal argument exception from JavaField.getByte(floatValue)");
		} catch (IllegalArgumentException e) {
			// expected
		}	
	}
	
	public void testLongValue() throws Exception {
		JavaObject thisJavaObject = getScenerioReference();
		JavaField longValueField = getJavaField(thisJavaObject, "longValue");

		try{
			byte aLongValue = longValueField.getByte(thisJavaObject);
			fail("Missing illegal argument exception from JavaField.getByte(longValue)");
		} catch (IllegalArgumentException e) {
			// expected
		}	
	}
	
	public void testIntValue() throws Exception {
		JavaObject thisJavaObject = getScenerioReference();
		JavaField intValueField = getJavaField(thisJavaObject, "intValue");

		try{
			byte aIntValue = intValueField.getByte(thisJavaObject);
			fail("Missing illegal argument exception from JavaField.getByte(intValue)");
		} catch (IllegalArgumentException e) {
			// expected
		}	
	}
	
	public void testCharValue() throws Exception {
		JavaObject thisJavaObject = getScenerioReference();
		JavaField charValueField = getJavaField(thisJavaObject, "charValue");

		try{
			byte aCharValue = charValueField.getByte(thisJavaObject);
			fail("Missing illegal argument exception from JavaField.getByte(charValue)");
		} catch (IllegalArgumentException e) {
			// expected
		}	
	}
	
	public void testShortValue() throws Exception {
		JavaObject thisJavaObject = getScenerioReference();
		JavaField shortValueField = getJavaField(thisJavaObject, "shortValue");

		try{
			byte aShortValue = shortValueField.getByte(thisJavaObject);
			fail("Missing illegal argument exception from JavaField.getByte(shortValue)");
		} catch (IllegalArgumentException e) {
			// expected
		}	
	}
	
	public void testBooleanValue() throws Exception {
		JavaObject thisJavaObject = getScenerioReference();
		JavaField booleanValueField = getJavaField(thisJavaObject, "booleanValue");
		
		try{
			byte aBooleanValue = booleanValueField.getByte(thisJavaObject);
			fail("Missing illegal argument exception from JavaField.getByte(booleanValue)");
		} catch (IllegalArgumentException e) {
			// expected
		}
	}
	
	public void testObjectReference() throws Exception {
		JavaObject thisJavaObject = getScenerioReference();
		JavaField objectReferenceField = getJavaField(thisJavaObject, "objectReference");
		
		try{
			byte anObjectReference = objectReferenceField.getByte(thisJavaObject);
			fail("Missing illegal argument exception from JavaField.getByte(objectReference)");
		} catch (IllegalArgumentException e) {
			// expected
		}
	}
}
