/*******************************************************************************
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 *   http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 ******************************************************************************/
package org.apache.kato.tck.scenario142.javaruntime;

import javax.tools.diagnostics.runtime.java.JavaField;
import javax.tools.diagnostics.runtime.java.JavaObject;

import org.apache.kato.tck.harness.TCKJavaRuntimeTestcase;
import org.apache.kato.tck.scenario142.javaruntime.SetupJavaField_getLong;


/**
 * Test JavaField.getLong() against various fields and values.
 *
 */
public class TestJavaField_getLong extends TCKJavaRuntimeTestcase {
	
	SetupJavaField_getLong setup=new SetupJavaField_getLong();
	
	static JavaObject thisObject = null;	
	public JavaObject getScenerioReference() {
		if (thisObject == null) {
			thisObject = super.getScenerioReference();
		}	
		return thisObject;		
	}
	
	public void testMaxLongValue() throws Exception {
		JavaObject thisJavaObject = getScenerioReference();
		JavaField maxLongField = getJavaField(thisJavaObject, "maxLong");
		
		long maxLongValue = maxLongField.getLong(thisJavaObject);
		
		assertEquals(setup.maxLong, maxLongValue);		
	}
	
	public void testMinLongValue() throws Exception {
		JavaObject thisJavaObject = getScenerioReference();
		JavaField minLongField = getJavaField(thisJavaObject, "minLong");
		
		long minLongValue = minLongField.getLong(thisJavaObject);
		
		assertEquals(setup.minLong, minLongValue);		
	}

	public void testStaticMinLongValue() throws Exception {
		JavaObject thisJavaObject = getScenerioReference();
		JavaField minLongField = getJavaField(thisJavaObject, "staticMinLong");
		
		long minLongValue = minLongField.getLong(null);
		
		assertEquals(SetupJavaField_getLong.staticMinLong, minLongValue);		
	}
	
	public void testZeroLongValue() throws Exception {
		JavaObject thisJavaObject = getScenerioReference();
		JavaField zeroLongField = getJavaField(thisJavaObject, "zeroLong");
		
		long zeroLongValue = zeroLongField.getLong(thisJavaObject);
		
		assertEquals(setup.zeroLong, zeroLongValue);		
	}
	
	public void testNegLongValue() throws Exception {
		JavaObject thisJavaObject = getScenerioReference();
		JavaField negLongField = getJavaField(thisJavaObject, "negLong");
		
		long negLongValue = negLongField.getLong(thisJavaObject);
		
		assertEquals(setup.negLong, negLongValue);		
	}	
	
	public void testPosLongValue() throws Exception {
		JavaObject thisJavaObject = getScenerioReference();
		JavaField posLongField = getJavaField(thisJavaObject, "posLong");
		
		long posLongValue = posLongField.getLong(thisJavaObject);
		
		assertEquals(setup.posLong, posLongValue);		
	}

	public void testDoubleValue() throws Exception {
		JavaObject thisJavaObject = getScenerioReference();
		JavaField doubleValueField = getJavaField(thisJavaObject, "doubleValue");

		try{
			long aDoubleValue = doubleValueField.getLong(thisJavaObject);
			fail("Missing illegal argument exception from JavaField.getLong(doubleValue)");
		} catch (IllegalArgumentException e) {
			// expected
		}	
	}
	
	public void testFloatValue() throws Exception {
		JavaObject thisJavaObject = getScenerioReference();
		JavaField floatValueField = getJavaField(thisJavaObject, "floatValue");

		try{
			long aFloatValue = floatValueField.getLong(thisJavaObject);
			fail("Missing illegal argument exception from JavaField.getLong(floatValue)");
		} catch (IllegalArgumentException e) {
			// expected
		}	
	}
	
	public void testIntValue() throws Exception {
		JavaObject thisJavaObject = getScenerioReference();
		JavaField intValueField = getJavaField(thisJavaObject, "intValue");

		try{
			long aIntValue = intValueField.getLong(thisJavaObject);
		} catch (IllegalArgumentException e) {
			fail("Unexpected illegal argument exception from JavaField.getLong(intValue)");
		}	
	}
	
	public void testByteValue() throws Exception {
		JavaObject thisJavaObject = getScenerioReference();
		JavaField byteValueField = getJavaField(thisJavaObject, "byteValue");

		try{
			long aByteValue = byteValueField.getLong(thisJavaObject);
		} catch (IllegalArgumentException e) {
			fail("Unexpected illegal argument exception from JavaField.getLong(byteValue)");
		}	
	}
	
	public void testCharValue() throws Exception {
		JavaObject thisJavaObject = getScenerioReference();
		JavaField charValueField = getJavaField(thisJavaObject, "charValue");

		try{
			long aCharValue = charValueField.getLong(thisJavaObject);
		} catch (IllegalArgumentException e) {
			fail("Unexpected illegal argument exception from JavaField.getLong(charValue)");
		}	
	}
	
	public void testShortValue() throws Exception {
		JavaObject thisJavaObject = getScenerioReference();
		JavaField shortValueField = getJavaField(thisJavaObject, "shortValue");

		try{
			long aShortValue = shortValueField.getLong(thisJavaObject);
		} catch (IllegalArgumentException e) {
			fail("Unexpected illegal argument exception from JavaField.getLong(shortValue)");
		}	
	}
	
	public void testBooleanValue() throws Exception {
		JavaObject thisJavaObject = getScenerioReference();
		JavaField booleanValueField = getJavaField(thisJavaObject, "booleanValue");
		
		try{
			long aBooleanValue = booleanValueField.getLong(thisJavaObject);
			fail("Missing illegal argument exception from JavaField.getLong(booleanValue)");
		} catch (IllegalArgumentException e) {
			// expected
		}
	}
	
	public void testObjectReference() throws Exception {
		JavaObject thisJavaObject = getScenerioReference();
		JavaField objectReferenceField = getJavaField(thisJavaObject, "objectReference");
		
		try{
			long anObjectReference = objectReferenceField.getLong(thisJavaObject);
			fail("Missing illegal argument exception from JavaField.getLong(objectReference)");
		} catch (IllegalArgumentException e) {
			// expected
		}
	}
}
