/*******************************************************************************
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 *   http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 ******************************************************************************/
package org.apache.kato.tck.scenario142.javaruntime;

import java.util.Iterator;

import javax.tools.diagnostics.image.CorruptDataException;
import javax.tools.diagnostics.image.DataUnavailable;
import javax.tools.diagnostics.image.MemoryAccessException;
import javax.tools.diagnostics.runtime.java.JavaField;
import javax.tools.diagnostics.runtime.java.JavaObject;
import javax.tools.diagnostics.runtime.java.JavaReference;

import org.apache.kato.tck.harness.TCKJavaRuntimeTestcase;


public class TestJavaObject_getReferences extends TCKJavaRuntimeTestcase {

	public void testMultiDimensionalArrayReferences() {
		try {
			JavaObject thisObject = getScenerioReference();
			JavaField multiDimArrayField = getJavaField(thisObject, "multiDimArray");
			Object multiDimArrayObject = multiDimArrayField.get(thisObject);
			if (multiDimArrayObject instanceof JavaObject) {
				JavaObject mdObject = (JavaObject)multiDimArrayObject;
				Iterator refs = mdObject.getReferences().iterator();
				while (refs.hasNext()) {
					Object next = refs.next();
					if (next instanceof JavaReference) {
						JavaReference ref = (JavaReference) next;
						try {
							Object target = ref.getTarget();
							if (target instanceof JavaObject) {
								JavaObject targetObject = (JavaObject) target;
								System.out.println(targetObject.getJavaClass().getName());
							} 
							System.out.println(ref.getDescription());
						} catch (DataUnavailable e) {
							e.printStackTrace();
						}
					} else {
						fail ("Found reference which is NOT an instance of JavaReference.");
					}
				}
			} else {
				fail("Wrong object type for multiDimArray.");
			}
		} catch (CorruptDataException e) {
			assertNotNull(e.getCorruptData());
			//allowed by the API
		} catch (MemoryAccessException mae) {
			assertNotNull(mae.getPointer());
			mae.printStackTrace();
		}
	}
	
}
