/*******************************************************************************
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 *   http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 ******************************************************************************/
package org.apache.kato.tests.junit;

import javax.tools.diagnostics.image.ImageFactory;

public class ImageFactoryTest extends AbstractImageTestcase
{
	/**
	 * Test method for 'javax.tools.diagnostics.image.ImageFactory.getDTFJMajorVersion()'
	 * 
	 * Ensures that the major version is at least 1
	 */
	public void testGetDTFJMajorVersion()
	{
		assertTrue(getFactory().getMajorVersion() >= 1);
	}

	/**
	 * Test method for 'javax.tools.diagnostics.image.ImageFactory.getDTFJMinorVersion()'
	 * 
	 * Ensures that the minor version is at least 0
	 */
	public void testGetDTFJMinorVersion()
	{
		assertTrue(getFactory().getMinorVersion() >= 0);
	}
	
	/**
	 * Test that the modification level is greater than or equal to 0
	 */
	
	public void testGetDTFJModificationLevel()
	{
		assertTrue(getFactory().getModificationLevel() >= 0);
	}
	
	/**
	 * Test that the modification level matches the implementation build no
	 * Note that the existence of the implementation version string is tested
	 * in the TestPackageInformation class
	 */
	
	public void testGetDTFJModificationLevelAsBuildNo()
	{
		String implementationVersion=ImageFactory.class.getPackage().getImplementationVersion();
		int buildNumber=0;
		try {
		buildNumber=Integer.parseInt(implementationVersion);
		} catch(NumberFormatException nfe) {
			buildNumber=-1;
		}
		assertTrue(getFactory().getModificationLevel()==buildNumber);
	}
}
