/*******************************************************************************
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 *   http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 ******************************************************************************/
package org.apache.kato.tests.junit;

import java.util.Iterator;

import javax.tools.diagnostics.image.DataUnavailable;
import javax.tools.diagnostics.image.ImageSection;
import javax.tools.diagnostics.runtime.java.JavaMethod;


public class ImageSectionTest extends AbstractImageTestcase
{
	private ImageSection _section = null;
	
	public  ImageSection defaultImageSection()
	{
		//for now we will just grab a compiled method section
		JavaMethod method = defaultJavaMethod();
		Iterator it = method.getCompiledSections().iterator();
		assertNotNull(it);
		assertTrue(it.hasNext());
		return (ImageSection) it.next();
	}
	
	
	protected void setUp() throws Exception
	{
		_section = defaultImageSection();
		super.setUp();
	}

	/**
	 * Test method for 'javax.tools.diagnostics.image.ImageSection.getBaseAddress()'
	 * 
	 * Ensure that we get a non-null base address
	 */
	public void testGetBaseAddress()
	{
		assertNotNull(_section.getBaseAddress());
	}

	/**
	 * Test method for 'javax.tools.diagnostics.image.ImageSection.getSize()'
	 * 
	 * Ensure that we get a >0 size
	 */
	public void testGetSize()
	{
		assertTrue(_section.getSize() > 0);
	}

	/**
	 * Test method for 'javax.tools.diagnostics.image.ImageSection.getName()'
	 * 
	 * Ensure that we get a non-null name
	 */
	public void testGetName()
	{
		assertNotNull(_section.getName());
	}

	/**
	 * Test method for 'javax.tools.diagnostics.image.ImageSection.isExecutable()'
	 * 
	 * Ensures that the method can be called
	 */
	public void testIsExecutable()
	{
		try {
			_section.isExecutable();
		} catch (DataUnavailable e) {
			//accepted by spec
		}
	}

	/**
	 * Test method for 'javax.tools.diagnostics.image.ImageSection.isReadOnly()'
	 * 
	 * Ensures that the method can be called
	 */
	public void testIsReadOnly()
	{
		try {
			_section.isReadOnly();
		} catch (DataUnavailable e) {
			//accepted by spec
		}
	}

	/**
	 * Test method for 'javax.tools.diagnostics.image.ImageSection.isShared()'
	 * 
	 * Ensures that the method can be called
	 */
	public void testIsShared()
	{
		try {
			_section.isShared();
		} catch (DataUnavailable e) {
			//accepted by spec
		}
	}
}
