/*******************************************************************************
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 *   http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 ******************************************************************************/
package org.apache.kato.tests.junit;

import java.util.Iterator;
import java.util.List;

import javax.tools.diagnostics.runtime.java.JavaHeap;


public class JavaHeapTest extends AbstractImageTestcase
{
	private JavaHeap _heap = null;
	
	protected void setUp() throws Exception
	{
		_heap = defaultJavaHeap();
		super.setUp();
	}

	/**
	 * Test method for 'javax.tools.diagnostics.runtime.java.JavaHeap.getSections()'
	 * 
	 * Ensure that the heap has at least one section
	 */
	public void testGetSections()
	{
		List iter = _heap.getSections();
		assertNotNull(iter);
		assertFalse(iter.isEmpty());
	}

	/**
	 * Test method for 'javax.tools.diagnostics.runtime.java.JavaHeap.getName()'
	 * 
	 * Ensure that the heap has a non-empty name
	 */
	public void testGetName()
	{
		String name = _heap.getName();
		assertNotNull(name);
		assertTrue(name.length() > 0);
	}

	/**
	 * Test method for 'javax.tools.diagnostics.runtime.java.JavaHeap.getObjects()'
	 * 
	 * Ensure that we return a non-empty iterator
	 */
	public void testGetObjects()
	{
		Iterator it = _heap.getObjects().iterator();
		assertNotNull(it);
		assertTrue(it.hasNext());
	}

	/**
	 * Verify that the equals call doesn't throw
	 */
	public void testEquals()
	{
		try{
			assertTrue(_heap.equals(_heap));
		} catch (Throwable t) {
			assertTrue(false);
		}
	}
	
	/**
	 * Verify that hashCode() doesn't throw and returns non-zero (technically zero is ok but it will be 
	 * flagged here to ensure that we aren't doing anything bad to create the hashcode)
	 */
	public void testHashCode()
	{
		try {
			assertTrue(0 != _heap.hashCode());
		} catch (Throwable t) {
			assertTrue(false);
		}
	}
}
