/*******************************************************************************
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 *   http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 ******************************************************************************/
package org.apache.kato.katoview;

import java.util.Vector;

public class Output {
	
	Vector outputChannels;
	long lastTimestamp = 0;
	String lastPrompt = "";
	String lastInput = "";

	public Output(){
		outputChannels = new Vector();
	}
	
	public void print(String outputString){
		for (int i = 0; i < outputChannels.size(); i++){
			((OutputChannel)outputChannels.elementAt(i)).print(outputString);
		}
	}
	
	public void printPrompt(String prompt){
		for (int i = 0; i < outputChannels.size(); i++){
			((OutputChannel)outputChannels.elementAt(i)).printPrompt(prompt);
		}
	}
	
	public void println(String outputString){
		for (int i = 0; i < outputChannels.size(); i++){
			((OutputChannel)outputChannels.elementAt(i)).println(outputString);
		}
	}
	
	public void error(String outputString){
		for (int i = 0; i < outputChannels.size(); i++){
			((OutputChannel)outputChannels.elementAt(i)).error(outputString);
		}
	}
	
	public void printInput(long timestamp, String prompt, String outputString){
		for (int i = 0; i < outputChannels.size(); i++){
			((OutputChannel)outputChannels.elementAt(i)).
				printInput(timestamp, prompt, outputString);
		}
		lastTimestamp = timestamp;
		lastPrompt = prompt;
		lastInput = outputString;
	}
	
	public void addChannel(OutputChannel channel){
		addChannel(channel, false);
	}
	
	public void addChannel(OutputChannel channel, boolean printLastInput){
		outputChannels.add(channel);
		if (printLastInput)
			channel.printInput(lastTimestamp, lastPrompt, lastInput);
	}
	
	public void removeFileChannel(){
		for (int i = 0; i < outputChannels.size(); i++){
			if ((OutputChannel)outputChannels.elementAt(i) instanceof FileOutputChannel)
			{
				((OutputChannel)outputChannels.elementAt(i)).close();
				outputChannels.removeElementAt(i);
			}
		}
	}
	
	public void setConsoleNoPrint(boolean noPrint){
		for (int i = 0; i < outputChannels.size(); i++){
			if ((OutputChannel)outputChannels.elementAt(i) instanceof ConsoleOutputChannel)
			{
				((ConsoleOutputChannel)outputChannels.elementAt(i)).setNoPrint(noPrint);
			}
		}
	}
	
	public void close() {
		for (int i = 0; i < outputChannels.size(); i++){
			((OutputChannel)outputChannels.elementAt(i)).close();
		}
	}
}
