/*******************************************************************************
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 *   http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 ******************************************************************************/
package org.apache.kato.katoview.commands;

import java.util.HashMap;
import java.util.Stack;

import javax.tools.diagnostics.image.Image;

import org.apache.kato.katoview.Output;
import org.apache.kato.katoview.commands.helpers.Utils;


public class FindNextCommand extends Command{
	public FindNextCommand(Output o){
		super(o, "findnext", "finds the next instance of the last string passed to \"find\"",
				"parameters: none\n\n" +
				"The findnext command is used in conjunction with find or findptr command " +
				"to continue searching for upcoming matches. It repeats " +
				"the previous find or findptr command (depending on which command is " +
				"most recently issued) starting from the last match."
				);
		child_commands = null;
	}
	
	public void doCommand(Stack args, Image loadedImage, HashMap properties){
		RootCommand rootCommand = (RootCommand)properties.get(Utils.RootCommand_OBJECT);
		if (null == rootCommand){
			out.error("ERROR - No mapping found for RootCommand object");
			return;
		}

		FindCommand.FindAttribute findAtt = (FindCommand.FindAttribute)properties.get(Utils.FIND_ATTRIBUTES);
		if (null == findAtt){
			out.println("No find command has been executed.");
			return;
		}
		
		String startAddress = Long.toHexString((findAtt.lastMatch + 1));
		String endAddress = Long.toHexString(findAtt.endAddress);
		
		Stack findCommandStack = Utils.constructStackFromString("find" + " " 
				+ findAtt.pattern + "," + startAddress + ","
				+ endAddress + "," + findAtt.boundary + ","
				+ findAtt.numBytesToPrint + "," + findAtt.numMatchesToDisplay);
		
		rootCommand.doCommand(findCommandStack, loadedImage, properties);
	}
}
