/*******************************************************************************
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 *   http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 ******************************************************************************/
package org.apache.kato.katoview.commands.infocommands;

import java.util.HashMap;
import java.util.Iterator;
import java.util.Stack;

import javax.tools.diagnostics.image.CorruptDataException;
import javax.tools.diagnostics.image.DataUnavailable;
import javax.tools.diagnostics.image.Image;
import javax.tools.diagnostics.runtime.ManagedRuntime;

import org.apache.kato.katoview.Output;
import org.apache.kato.katoview.commands.Command;
import org.apache.kato.katoview.commands.helpers.Utils;


public class InfoSystemCommand extends Command {

	public InfoSystemCommand(Output o){
		super(o, "system", "displays information about the system the core dump is from",
				"parameters: none\n\n" +
				"prints the following information about the system the core " +
				"dump is from:\n" +
				"  - operating system\n" +
				"  - amount of memory\n" +
				"  - virtual machine(s) present\n"
				);
		child_commands = null;
	}
	
	public void doCommand(Stack args, Image loadedImage, HashMap properties){
		if (!args.isEmpty())
		{
			out.error("\"info system\" command does not take any parameters");
		}
		else
		{
			Iterator itRuntime = Utils.getRuntimes(loadedImage);
			ManagedRuntime mr;
			String output = "";
			try {
				output += "\nSystem:\t\t";
				output += loadedImage.getSystemType() + "\n";
				output += "System Memory:\t";
				output += loadedImage.getInstalledMemory() + " bytes\n";
				output += "Virtual Machine(s):\n";
				
				int count = 1;
				while (itRuntime.hasNext()) {
					mr = (ManagedRuntime)itRuntime.next();
					output += "\tRuntime #" + count + ":\n\t\t";
					output += mr.getVersion() + "\n";
					count++;
				}
			} catch (DataUnavailable d) {
				
			} catch (CorruptDataException e) {
				
			}
			
			if (!output.equals(""))
				out.println(output);
			else
				out.error("error occurred while processing \"info system\"");
		}
	}
}
