/*******************************************************************************
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 *   http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 ******************************************************************************/
package org.apache.kato.katoview.commands.xcommands;

import java.util.HashMap;
import java.util.Stack;
import java.util.Iterator;

import javax.tools.diagnostics.image.CorruptDataException;
import javax.tools.diagnostics.image.Image;
import javax.tools.diagnostics.image.ImageAddressSpace;
import javax.tools.diagnostics.image.ImagePointer;
import javax.tools.diagnostics.image.MemoryAccessException;

import org.apache.kato.katoview.Output;
import org.apache.kato.katoview.commands.Command;
import org.apache.kato.katoview.commands.helpers.Utils;


public class XDCommand extends Command {

	public XDCommand(Output o){
		super(o, "d", "displays the integer at the specified address",
				"parameters: 0x<addr>\n\n" +
				"Displays the integer at the specified address, adjusted for the " +
				"endianness of the architecture this dump file is from.\n\n" +
				"Note: This command uses the number of items and unit size passed to " +
				"it by the \"x/\" command.\n"
		);
		child_commands = null;
	}
	
	public void doCommand(Stack args, Image loadedImage, HashMap properties)
	{
		int numUnits = ((Integer)args.pop()).intValue();
		int unitSize = ((Integer)args.pop()).intValue();
		
		if (args.isEmpty())
		{
			out.error("\"x/d\" command takes exactly one parameter");
			return;
		}

		String param = (String)args.pop();
		if (!args.isEmpty())
		{
			out.error("\"x/d\" only takes one parameter");
			return;
		}
		Long address;
		
		address = Utils.longFromStringWithPrefix(param);
		if (null == address)
		{
			out.error("invalid hex address specified; address must be specified as "
					+ "\"0x<hex_address>\"");
			return;
		}

		out.print("\n");
		
		for (int index = 0; index < numUnits; index++)
		{
			boolean found = false;
			Iterator itImageAddressSpace = loadedImage.getAddressSpaces().iterator();
			long currAddr = address.longValue() + (index * unitSize);
			
			out.print("\t");
			out.print(Utils.toHex(currAddr));
			out.print(": ");
			
			while (itImageAddressSpace.hasNext() && !found)
			{
				ImageAddressSpace ias = (ImageAddressSpace)itImageAddressSpace.next();
				ImagePointer ip = ias.getPointer(currAddr);
				
				byte b = 0;
				short s = 0;
				int i = 0;
				long l = 0;
				try {
					switch (unitSize)
					{
					case 1:
						b = ip.getByteAt(0);
						break;
					case 2:
						s = ip.getShortAt(0);
						break;
					case 4:
						i = ip.getIntAt(0);
						break;
					case 8:
						l = ip.getLongAt(0);
						break;
					}
					
					found = true;
				} catch (CorruptDataException e) {
					found = false;
				} catch (MemoryAccessException e) {
					found = false;
				}

				if (found)
				{
					switch (unitSize)
					{
					case 1:
						out.print(Byte.toString(b));
						break;
					case 2:
						out.print(Short.toString(s));
						break;
					case 4:
						out.print(Integer.toString(i));
						break;
					case 8:
						out.print(Long.toString(l));
						break;
					}
				}
			}
			
			if (!found)
			{
				out.print("<address not found in any address space>");
			}
			out.print("\n");
		}
		
		out.print("\n");
	}
	
}
